import requests
import site_conn_check.sites_db as sites_db

def getStatus(url):
    try:
        response = requests.head(url, timeout=1)
        response_time = response.elapsed.total_seconds() * 1000
        return response.status_code, response_time
    except requests.exceptions.ConnectTimeout:
        return "Timed Out", 0


def updateDb():
    sites = sites_db.Database()
    with sites as db:
        urls = sites.get_urls()
        urls = [i[0] for i in urls] 
    with sites as db:
        for url in urls:
            status_code, response_time = getStatus(url)
            if status_code is not None:
                sites.update_status_code(url, status_code, response_time)





import sqlite3
import os
import pandas as pd

class Database():
    def __init__(self):
        self.connection = None
        self.cursor = None
    
    def __enter__(self):
        self.connection = sqlite3.connect('sites.db', check_same_thread=False)
        self.cursor = self.connection.cursor()
        self.cursor.execute('''
            CREATE TABLE IF NOT EXISTS sites (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                name TEXT NOT NULL,
                url TEXT NOT NULL,
                response INT,
                status TEXT,
                UNIQUE(name,url)
            )
        ''')
        self.connection.commit()
    
    def add_site(self, name, url):
        self.cursor.execute('SELECT name FROM sites WHERE name=?', (name,))
        result_name = self.cursor.fetchone()
        self.cursor.execute('SELECT url FROM sites WHERE url=?', (url,))
        result_url = self.cursor.fetchone()

        if result_name or result_url:
            return False
        if not result_url and not result_name:
            self.cursor.execute('''
                INSERT INTO sites (name, url)
                VALUES (?, ?)
            ''', (name, url))
            return True
    
    def remove_site(self, name):
        self.cursor.execute('SELECT name FROM sites WHERE name =?', (name,))
        result = self.cursor.fetchone()
        if result:
            self.cursor.execute('''
                DELETE FROM sites
                WHERE name = ?
            ''', (name,))
            return True
        else:
            return False

    def get_sites(self, detailed=False):
        if not isinstance(detailed, bool):
            raise TypeError('detailed argument must be a bool')
        if detailed:
            sites = pd.read_sql_query('SELECT name, url, response, status FROM sites', self.connection)
            return sites
        else:
            sites = pd.read_sql_query('SELECT name, url FROM sites', self.connection)
            return sites
        

    def get_urls(self):
        self.cursor.execute('''
            SELECT url FROM sites
        ''')
        return self.cursor.fetchall()
    
    def update_site(self, name, url):
        self.cursor.execute('SELECT name FROM sites WHERE name =?', (name,))
        exists = self.cursor.fetchone()
        if exists:
            self.cursor.execute('''
                        UPDATE sites
                        SET url = ?
                        WHERE name = ?
                    ''', (url, name))
            return True
        else:
            return False

    def update_status_code(self, url, code, response_time):
        self.cursor.execute('''
            UPDATE sites
            SET status = ?, response =?
            WHERE url = ?
        ''', (code, response_time, url))

    def get_number_of_rows(self):
        self.cursor.execute('''
            SELECT COUNT(*) FROM sites
        ''')
        return self.cursor.fetchone()[0]
    
    def get_status_code(self, name):
        self.cursor.execute('''
            SELECT status FROM sites
            WHERE name = ?
        ''', (name,))
        return self.cursor.fetchone()[0]
    
    def delete_database(self):
        self.connection.close()
        os.remove('sites.db')

    def update_response(self, url, response):

        self.cursor.execute('''
            UPDATE sites
            SET response = ?
            WHERE url = ?
        ''', (response, url))

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.connection.commit()
        self.connection.close()


from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='site_conn_check',
    version='1.0.3',
        zip_safe=False,
    author="Joshua Abbey",
    author_email="Joshuaabbey2022@gmail.com",
    description="Package to check status code and response time of websites",
    url="https://github.com/rxxyxd/site-conn-check",
    long_description=long_description,
    long_description_content_type ='text/markdown',
    packages=["site_conn_check"],
    install_requires=[
        'certifi==2023.5.7',
        'charset-normalizer==3.1.0',
        'decorator==5.1.1',
        'idna==3.4',
        'numpy==1.24.3',
        'pandas==2.0.2',
        'python-dateutil==2.8.2',
        'pytz==2023.3',
        'requests==2.31.0',
        'six==1.16.0',
        'tabulate==0.9.0',
        'tzdata==2023.3',
        'validators==0.20.0',
    ],
    license='LICENCE',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            # command 
            'site_conn_check = site_conn_check.main:main',
        ]
    }
)


import os
from tabulate import tabulate
import argparse
import validators

import site_conn_check.sites_db as sites_db
import site_conn_check.conn_check as conn_check

def main():
    db = sites_db.Database()

    parser = argparse.ArgumentParser(prog="main.py", description="This is a program to retrieve the status and response time of sites in a database")

    parser.add_argument("-s", "--sites", help="Lists all sites in database", action="store_true")
    parser.add_argument("--run", help="Adds a site to the database", action="store_true")
    parser.add_argument("-d", "--delete", help="Deletes the Database", action="store_true")

    subparser = parser.add_subparsers(title="command COMMAND", dest="command", help="sub-command for help")

    add_parser = subparser.add_parser("add", help="Add a new site to database", )
    add_parser.add_argument("-n", "--name", help="Name of the site", required=True)
    add_parser.add_argument("-u", "--url", help="URL of the site", required=True)

    remove_parser = subparser.add_parser("remove", help="Remove a site from database")
    remove_parser.add_argument("-n", "--name", help="Name of the site", required=True)

    update_parser = subparser.add_parser("update", help="Update a site in database")
    update_parser.add_argument("-n", "--name", help="Name of the site", required=True)
    update_parser.add_argument("-u", "--url", help="URL of the site", required=True)

    args = parser.parse_args()
    with db as _:
    
        if args.sites:
            print(tabulate(db.get_sites(), showindex="never", headers="keys", tablefmt='psql'), end="\r")
            print("\n")
        elif args.run:
            os.system("cls" if os.name == "nt" else "clear")
            try:
                while True:
                    conn_check.updateDb()
                    sitesList = db.get_sites(True)
                    print(tabulate(sitesList, showindex="never", headers="keys", tablefmt='psql'), end="\r")
                    len_sites = len(sitesList.index)
                    print('\033[1A'*(len_sites+3),end="\x1b[2K")
            except KeyboardInterrupt:
                os.system("cls" if os.name == "nt" else "clear")
                print("Exiting...")
                
        elif args.delete:
            db.delete_database()
            print("Database deleted successfully")
        elif args.command == "add":
            valid = validators.url(args.url)
            if valid:
                success = db.add_site(args.name.capitalize(), args.url.lower())
                if success:
                    print("Site added successfully")
                else:
                    print("Supplied URL or Name already exists")
            else:
                print("URL is not valid. Please enter a valid URL")
        elif args.command == "remove":
            success = db.remove_site(args.name.capitalize())
            if success:
                print("Site removed successfully")
            else: 
                print("Site not found")
        elif args.command == "update":
            valid = validators.url(args.url)
            if valid:
                success = db.update_site(args.name.capitalize(), args.url.lower())
                if success:
                    print("Site updated successfully")
                else:
                    print("Site not found")
            else:
                print("URL is not valid. Please enter a valid URL")

if __name__ == "__main__":
    main()